/*
  Copyright (C) 2001 Kimmo Pekkola

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
/*
  $Header: //rainbox/cvsroot/Rainmeter/Plugin/MeasureNetIn.cpp,v 1.4 2001/09/01 13:00:10 rainy Exp $

  $Log: MeasureNetIn.cpp,v $
  Revision 1.4  2001/09/01 13:00:10  rainy
  Slight changes in the interface. The value is now measured only once if possible.

  Revision 1.3  2001/08/19 09:14:55  rainy
  Added support for value invert.
  Also the CMeasure's ReadConfig is executed.

  Revision 1.2  2001/08/12 15:44:49  Rainy
  Adjusted Update()'s interface.
  Net throughput is now measured in bytes per second and not bits.

  Revision 1.1.1.1  2001/08/11 10:58:19  Rainy
  Added to CVS.

*/

#include "MeasureNetIn.h"

UINT CMeasureNetIn::c_Value = 0;
int CMeasureNetIn::c_Counter = -1;

CMeasureNetIn::CMeasureNetIn() : CMeasureNet()
{
}

CMeasureNetIn::~CMeasureNetIn()
{
}

bool CMeasureNetIn::Update(CMeterWindow& meterWindow, int counter)
{
	static bool firstTime = true;
	static UINT inOctets;
	int i;

	if(CMeasure::Update(meterWindow, counter)) return true;
	c_Counter = counter;

	UpdateTable(counter);

	if(c_Table == NULL) return false;

	if(!firstTime) 
	{
		c_Value = 0;
		for(i = 0; i < c_NumOfTables; i++)
		{
			// Ignore the loopback
			if(strcmp((char*)c_Table->table[i].bDescr, "MS TCP Loopback interface") != 0)
			{
				c_Value += c_Table->table[i].dwInOctets;
			}
		}
		UINT tmpValue = c_Value;
		c_Value -= inOctets;
		inOctets = tmpValue;
	}
	else
	{
		inOctets = 0;
		for(i = 0; i < c_NumOfTables; i++)
		{
			// Ignore the loopback
			if(strcmp((char*)c_Table->table[i].bDescr, "MS TCP Loopback interface") != 0)
			{
				inOctets += c_Table->table[i].dwInOctets;
			}
		}

		firstTime = false;
	}

	return false;
}

void CMeasureNetIn::ReadConfig(const char* filename, const char* section)
{
	CMeasure::ReadConfig(filename, section);

	int value = GetPrivateProfileInt(section, "NetInSpeed", 0, filename);

	if(value == 0)
	{
		m_MaxValue = 1;
		m_LogMaxValue = true;
	}
	else
	{
		m_MaxValue = value / 8;
	}
}
